<?php
/*--------------------------------------------------------------------------------------------------
    ClassBuilder.php 2019-08-30
    Gambio GmbH
    http://www.gambio.de
    Copyright (c) 2019 Gambio GmbH
    Released under the GNU General Public License (Version 2)
    [http://www.gnu.org/licenses/gpl-2.0.html]
    --------------------------------------------------------------------------------------------------
 */

namespace Gambio\StyleEdit\Core\BuildStrategies;

use Exception;
use Gambio\StyleEdit\Core\SingletonPrototype;
use Psr\Container\ContainerInterface;

/**
 * Class ClassBuilder
 * @package Gambio\StyleEdit\Core\BuildStrategies
 */
abstract class ClassBuilder implements ContainerInterface
{
    /**
     *
     * @var SingletonPrototype
     */
    private static $instance;
    protected      $objectList = [];
    
    
    /**
     * SingletonPrototype constructor.
     */
    private function __construct()
    {
        $this->initialize();
    }
    
    
    /**
     * return a instance for the $className param
     *
     * @param string $className
     *
     * @return bool|mixed
     * @throws Exception
     */
    public function get($className)
    {
        if (array_key_exists($className, $this->objectList)) {
            $instance = $this->objectList[$className];
            
            if ((is_object($instance) && ($instance instanceof Closure))//unnamed function
                || (!is_string($instance) && is_callable($instance)))//callable array
            {
                $instance = $instance();
                if (!($instance instanceof AlwaysNewInterface)) {
                    $this->setUp($className, $instance);
                }
            }
            
            if ($instance instanceof NotCloneableInterface) {
                return $instance;
            } else {
                return clone $instance;
            }
        } elseif (class_exists($className)) {
            
            $reflection  = new \ReflectionClass($className);
            $inputParams = [];
            
            if ($constructor = $reflection->getConstructor()) {
                $params = $constructor->getParameters();
                if (count($params)) {
                    foreach ($params AS $param) {
                        $paramValue = $this->get($param->getClass()->name);
                        if (!$paramValue) {
                            
                            throw new Exception('Cannot find a class: ' . $param->getClass()->name);
                        }
                        $inputParams[] = $paramValue;
                    }
                }
            }
            
            $instance = $reflection->newInstanceArgs($inputParams);
            if (!($instance instanceof AlwaysNewInterface)) {
                $this->setUp($className, $instance);
            }
            
            return clone $instance;
        }
        
        return false;
    }
    
    
    /**
     * return the singleton instance for the prototype class
     * @return SingletonPrototype
     */
    public static function instance(): SingletonPrototype
    {
        if (self::$instance === null) {
            self::$instance = new SingletonPrototype();
        }
        
        return self::$instance;
    }
    
    
    /**
     * Setup an instance of a specified class.
     *
     * @param string      $className
     * @param             $object
     */
    public function setUp(string $className, $object): void
    {
        $this->objectList[$className] = $object;
    }
    
    
    /**
     * Remove a prototype from the arrayList
     *
     * @param string $objectReference
     */
    public function remove(string $objectReference): void
    {
        if (array_key_exists($objectReference, $this->objectList)) {
            unset($this->objectList[$objectReference]);
        }
    }
    
    
    /**
     * Returns true if the container can return an entry for the given identifier.
     * Returns false otherwise.
     *
     * `has($id)` returning true does not mean that `get($id)` will not throw an exception.
     * It does however mean that `get($id)` will not throw a `NotFoundExceptionInterface`.
     *
     * @param string $id Identifier of the entry to look for.
     *
     * @return bool
     */
    public function has($id)
    {
        return array_key_exists($id, $this->objectList) || class_exists($id);
    }
    
}