<?php
/* --------------------------------------------------------------
  WysiwygSaveCommand.php 2019-08-02
  Gambio GmbH
  http://www.gambio.de
  Copyright (c) 2019 Gambio GmbH
  Released under the GNU General Public License (Version 2)
  [http://www.gnu.org/licenses/gpl-2.0.html]
  --------------------------------------------------------------*/

namespace Gambio\StyleEdit\Core\Components\Wysiwyg\Commands;

use ContentNotFoundException;
use ContentReadServiceInterface;
use ContentStatus;
use ContentValueObjectFactory;
use ContentWriteServiceInterface;
use ElementPosition;
use Exception;
use Gambio\StyleEdit\Core\Language;
use Gambio\StyleEdit\Core\Options\Commands\AbstractSaveCommand;
use Gambio\StyleEdit\Core\SingletonPrototype;
use Gambio\StyleEdit\Core\Components\ContentManager\Entities\AbstractContentManagerOption;
use Gambio\StyleEdit\Core\Components\Wysiwyg\Entities\WysiwygOption;
use Gambio\StyleEdit\Core\Components\Wysiwyg\Entities\WysiwygOptionValue;
use InfoElementContent;
use InfoElementContentBuilder;
use LanguageCode;
use ReflectionException;
use UnfinishedBuildException;

/**
 * Class WysiwygSaveCommand
 */
abstract class ContentManagerSaveCommand extends AbstractSaveCommand
{
    /**
     * @var ContentReadServiceInterface
     */
    protected $contentReadService;
    
    /**
     * @var ContentWriteServiceInterface
     */
    protected $contentWriteService;
    
    /**
     * @var AbstractContentManagerOption
     */
    protected $option;
    
    /**
     * @var string
     */
    protected $defaultTitle;
    
    
    /**
     * WysiwygSaveCommand constructor.
     * @throws Exception
     */
    public function __construct()
    {
        $this->contentWriteService = SingletonPrototype::instance()->get(ContentWriteServiceInterface::class);
        $this->contentReadService  = SingletonPrototype::instance()->get(ContentReadServiceInterface::class);
    }
    
    
    /**
     * Execute the command
     *
     * @throws UnfinishedBuildException
     * @throws ReflectionException
     */
    public function execute(): void
    {
        if (is_a($this->option, AbstractContentManagerOption::class)) {
            
            $contentManagerEntryExists = true;
            //  is this content zone already saved?
            try {
                $this->contentReadService->findById($this->option->contentGroup());
            } catch (ContentNotFoundException $notFoundException) {
                $contentManagerEntryExists = false;
            }
            
            $infoElementContent = $this->createInfoElementContent();
            
            $method = $contentManagerEntryExists ? 'updateInfoElementContent' : 'storeInfoElementContent';
            
            $this->contentWriteService->$method($infoElementContent);
        }
    }
    
    
    /**
     * @return InfoElementContent
     * @throws UnfinishedBuildException
     * @throws ReflectionException
     */
    protected function createInfoElementContent(): InfoElementContent
    {
        $headingsContent = $textsContent = $titlesContent = [];
        
        /** @var LanguageCode $languageCode */
        foreach (ContentValueObjectFactory::languageProvider()->getActiveCodes() as $languageCode) {
            
            $languageId = ContentValueObjectFactory::languageProvider()->getIdByCode($languageCode);
            $language   = SingletonPrototype::instance()->get(Language::class, $languageCode);
            
            $valueObject = $this->option->value($language)[strtolower($languageCode)];
            
            /** @var WysiwygOptionValue $valueObject */
            $headingsContent[] = ['languages_id' => $languageId, 'content_heading' => ''];
            
            $title = $valueObject->title();
            $title = $title === '' ? $this->defaultTitle() : $title;
            
            $titlesContent[] = ['languages_id' => $languageId, 'content_title' => $title];
            $textsContent[]  = ['languages_id' => $languageId, 'content_text' => $valueObject->value()];
        }
        
        $headings = ContentValueObjectFactory::createContentHeadingCollection($headingsContent);
        $texts    = ContentValueObjectFactory::createContentTextCollection($textsContent);
        $titles   = ContentValueObjectFactory::createContentTitleCollection($titlesContent);
        
        if ($this->option->contentGroup() === null) {
            
            $this->option->setContentGroup($this->contentReadService->nextContentGroupId());
        }
        
        return InfoElementContentBuilder::create()
            ->usingId($this->option->contentGroup())
            ->usingStatus(new ContentStatus(true))
            ->inPosition(ElementPosition::createForStyleEdit())
            ->usingHeadings($headings)
            ->usingTexts($texts)
            ->usingTitles($titles)
            ->build();
    }
    
    
    /**
     * Execute the command
     */
    public function rollback(): void
    {
        // TODO: Implement rollback() method.
    }
    
    
    /**
     * @return string
     * @throws Exception
     */
    protected function defaultTitle(): string
    {
        if ($this->defaultTitle === null) {
            
            $hash = date('Y-m-d H:i:s') . random_int(0, 100);
            $hash = md5($hash);
            $hash = substr($hash, 0, 6);
            
            $this->defaultTitle = 'StyleEdit-' . $hash;
        }
        
        return $this->defaultTitle;
    }
}