<?php
/* --------------------------------------------------------------
   ThemeConfiguration.php 2019-07-09
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2016 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

namespace Gambio\StyleEdit\Core\Components\Theme\Entities;

use FileNotFoundException;
use Gambio\StyleEdit\Core\Components\Theme\Entities\Interfaces\CurrentThemeInterface;
use Gambio\StyleEdit\Core\Components\Variant\Services\VariantService;
use Gambio\StyleEdit\Core\Language;
use Gambio\StyleEdit\Core\Services\ThemeValidator;
use LanguageService;
use Gambio\StyleEdit\Core\Options\Entities\ConfigurationCategory;
use Gambio\StyleEdit\Core\Repositories\ConfigurationRepository;
use Gambio\StyleEdit\Core\SingletonPrototype;
use LanguageHelperInterface;
use stdClass;
use ThemeControl;

/**
 * Class ThemeConfiguration
 * @package Gambio\StyleEdit\Core\Components\Theme
 */
class ThemeConfiguration implements CurrentThemeInterface
{
    /**
     * @var string
     */
    protected $id = '';
    
    /**
     * @var array of Language
     */
    protected $languages = [];
    
    /**
     * @var string
     */
    protected $title = '';
    
    /**
     * @var string
     */
    protected $thumbnail = '';
    
    /**
     * @var string
     */
    protected $author = '';
    
    /**
     * @var string
     */
    protected $version = '';
    
    /**
     * @var array
     */
    protected $colorPalette;
    
    /**
     * @var string|null
     */
    protected $extendsOf = '';
    
    /**
     * @var string
     */
    protected $path = '';
    
    /**
     * @var array
     */
    protected $children = [];
    
    /**
     * @var bool
     */
    protected $active = false;
    
    /**
     * @var bool
     */
    protected $preview = false;
    
    /**
     * @var ConfigurationCategory
     */
    protected $basics;
    
    /**
     * @var ConfigurationCategory
     */
    protected $areas;
    
    /**
     * @var ConfigurationCategory
     */
    protected $styles;
    
    /**
     * @var LanguageService
     */
    protected $languageService;
    
    
    /** Creates a themeConfiguration instance with all the related data
     *
     * @param stdClass $themeConfig
     * @param string   $path
     *
     * @return ThemeConfiguration
     * @throws \Exception
     */
    public static function ThemeConfigurationWithJsonAndThemePathExtended(
        stdClass $themeConfig,
        string $path
    ): ThemeConfiguration {
        $result = self::ThemeConfigurationWithJsonAndThemePath($themeConfig, $path);
        
        $configurationRepository = SingletonPrototype::instance()->get(ConfigurationRepository::class);
        $variantService          = SingletonPrototype::instance()->get(VariantService::class);
        
        $result->languageSpecificContent($themeConfig);
        
        if (isset($themeConfig->config)) {
            if (isset($themeConfig->config->basics) && $themeConfig->config->basics) {
                
                $result->basics = ConfigurationCategory::createFromJsonObject($themeConfig->config->basics,
                                                                              $variantService,
                                                                              $configurationRepository);
            }
            
            if (isset($themeConfig->config->areas) && $themeConfig->config->areas) {
                $result->areas = ConfigurationCategory::createFromJsonObject($themeConfig->config->areas,
                                                                             $variantService,
                                                                             $configurationRepository);
            }
            
            if (isset($themeConfig->colorPalette)) {
                $result->colorPalette = $themeConfig->colorPalette;
            }
            
            if (isset($themeConfig->config->styles) && $themeConfig->config->styles) {
                $result->styles = ConfigurationCategory::createFromJsonObject($themeConfig->config->styles,
                                                                              $variantService,
                                                                              $configurationRepository);
            }
        }
        /**
         * @var LanguageHelperInterface
         */
        $helper = SingletonPrototype::instance()->get(LanguageHelperInterface::class);
        
        foreach ($helper->getActiveLanguageCodes() as $languageId => $code) {
            $result->languages[] = new Language($code->asString(), $languageId);
        }
        
        return $result;
    }
    
    
    /** Simplified ThemeConfiguration for listing purposes
     *
     * @param $themeConfig
     * @param $path
     *
     * @return ThemeConfiguration
     * @throws \Exception
     */
    public static function ThemeConfigurationWithJsonAndThemePath(
        stdClass $themeConfig,
        string $path
    ): ThemeConfiguration {
    
        $themeControl      = self::themeControll();
        $result            = new self;
        $result->id        = (string)$themeConfig->id;
        $result->title     = (string)isset($themeConfig->title) ? (string)$themeConfig->title : (string)$themeConfig->id;
        $result->thumbnail = (string)$themeConfig->thumbnail;
        $result->author    = (string)$themeConfig->author;
        $result->preview   = (bool)$themeConfig->preview;
        $result->version   = (string)$themeConfig->version;
        $result->extendsOf = isset($themeConfig->extends) ? (string)$themeConfig->extends : null;
        $result->path      = $path;
        $result->active    = $result->id === $themeControl->getCurrentTheme();
        $result->children  = $themeConfig->children ?? [];
        
        return $result;
    }
    
    
    /**
     * @return ThemeControl
     * @throws \Exception
     */
    protected static function themeControll(): ThemeControl
    {
        return SingletonPrototype::instance()->get(ThemeControl::class);
    }
    
    
    /**
     * @param \Gambio\StyleEdit\Core\Components\Theme\Entities\ThemeConfiguration $parent
     * @param                                                                     $themeConfig
     * @param                                                                     $id
     * @param                                                                     $title
     *
     * @return \Gambio\StyleEdit\Core\Components\Theme\Entities\ThemeConfiguration
     * @throws FileNotFoundException
     */
    public static function cloneThemeConfiguration(ThemeConfiguration $parent, $themeConfig, $id, $title)
    {
        $parentPath                         = explode(DIRECTORY_SEPARATOR, $parent->path());
        $parentPath[count($parentPath) - 1] = $id;
        
        $data              = array_merge($parent->toArray(), get_object_vars($themeConfig));
        $result            = ThemeConfiguration::ThemeConfigurationWithJsonAndThemePath((object)$data,
                                                                                        implode(DIRECTORY_SEPARATOR,
                                                                                                $parentPath));
        $result->id        = $id;
        $result->title     = $title;
        $result->extendsOf = $parent->id();
        $result->active    = false;
        
        return $result;
    }
    
    
    /**
     * @return string
     */
    public function path()
    {
        return (string)$this->path;
    }
    
    
    /**
     * @return bool
     */
    public function preview(): bool
    {
        return $this->preview && strpos($this->id(), '_preview');
    }
    
    
    /**
     * @return array
     * @throws FileNotFoundException
     */
    public function toArray()
    {
        $themeData          = [];
        $themeData['id']    = $this->id();
        $themeData['title'] = $this->title();
        
        if ($this->extendsOf()) {
            $themeData['extends'] = $this->extendsOf();
        }
        
        if ($this->preview()) {
            $themeData['preview'] = $this->preview();
        }
        
        $themeData['thumbnail'] = $this->absolutePathForThumbnail();
        $themeData['author']    = $this->author();
        $themeData['version']   = $this->version();
        $themeData['removable'] = $this->removable();
        if ($this->colorPalette()) {
            $themeData['colorPalette'] = $this->colorPalette();
        }
        $themeData['languages'] = $this->languages();
        
        if ($this->basics() || $this->areas()) {
            $themeData['config'] = [];
            if ($this->basics()) {
                $themeData['config']['basics'] = $this->basics();
            }
            
            if ($this->areas()) {
                $themeData['config']['areas'] = $this->areas();
            }
            
            if ($this->styles()) {
                $themeData['config']['styles'] = $this->styles();
            }
        }
        
        if ($this->active()) {
            $themeData['active'] = $this->active();
        }
        
        $themeData['editable'] = $this->themeIsEditable();
        
        return $themeData;
    }
    
    
    /**
     * @return bool
     * @throws FileNotFoundException
     */
    protected function themeIsEditable(): bool
    {
        return $this->preview() === false && ThemeValidator::for($this->getRelativePathFromThemes())->canBeOpenedInStyleEdit4();
    }
    
    /**
     * @return string|null
     */
    protected function getRelativePathFromThemes(): ?string
    {
        $result = str_replace(SHOP_ROOT . 'themes/', '', $this->path());
        
        if (substr($result, -1) === DIRECTORY_SEPARATOR) {
            
            return substr($result, 0, -1);
        }
        
        return $result;
    }
    
    /**
     * @return string
     */
    public function id()
    {
        return $this->id;
    }
    
    
    /**
     * @return string
     */
    public function title()
    {
        return $this->title;
    }
    
    
    /**
     * @return string|null
     */
    public function extendsOf()
    {
        return $this->extendsOf;
    }
    
    
    /**
     * @return string
     */
    public function thumbnail()
    {
        return $this->thumbnail;
    }
    
    
    /**
     * @return string
     */
    public function author(): ?string
    {
        return $this->author;
    }
    
    
    /**
     * @return string
     */
    public function version(): ?string
    {
        return $this->version;
    }
    
    
    /**
     * @return array
     */
    public function colorPalette(): ?array
    {
        return $this->colorPalette;
    }
    
    
    /**
     * @return array
     */
    public function languages(): array
    {
        return $this->languages;
    }
    
    
    /**
     * @return bool
     */
    public function removable()
    {
        return !(count($this->children) > 0 || in_array($this->id, $this->getGambioThemeIds(), false) || $this->active) || $this->preview();
    }
    
    
    /**
     * @return bool
     */
    public function active()
    {
        return $this->active;
    }
    
    
    /**
     * @return array
     */
    protected function getGambioThemeIds()
    {
        return [
            'Honeygrid',
            'Childgrid',
            'Grandgrid'
        ];
    }
    
    
    /**
     * @return array
     */
    public function &children()
    {
        return $this->children;
    }
    
    
    /**
     * @return ConfigurationCategory
     */
    public function basics(): ?ConfigurationCategory
    {
        return $this->basics;
    }
    
    
    /**
     * @return ConfigurationCategory
     */
    public function areas(): ?ConfigurationCategory
    {
        return $this->areas;
    }
    
    
    /**
     * Language Specific Content
     *
     * @param stdClass $themeConfiguration
     *
     * @throws \ReflectionException
     */
    protected function languageSpecificContent(stdClass $themeConfiguration): void
    {
        $this->languageService = SingletonPrototype::instance()->get(LanguageService::class);
        
        $bilingualProperties = ['title'];
        
        foreach ($bilingualProperties as $property) {
            
            $this->$property = $this->languageService->translate($this->$property);
        }
        
        $this->translateCategories($themeConfiguration->config->basics);
        $this->translateCategories($themeConfiguration->config->areas);
    }
    
    
    /**
     * Recursive function for the language specific content
     *
     * @param $content stdClass|array
     */
    protected function translateCategories($content): void
    {
        if (isset($content->categories)) {
            foreach ($content->categories as &$category) {
                if (isset($category->title) && $category->title !== null) {
                    $category->title = $this->languageService->translate($category->title);
                }
                
                if (isset($category->categories) || isset($category->fieldsets)) {
                    $this->translateCategories($category);
                }
            }
            
            unset($category);
        } elseif (isset($content->fieldsets)) {
            foreach ($content->fieldsets as &$fieldset) {
                if (isset($fieldset->title) && $fieldset->title !== null) {
                    $fieldset->title = $this->languageService->translate($fieldset->title);
                }
                if (isset($fieldset->options)) {
                    $this->translateCategories($fieldset);
                }
            }
            
            unset($fieldset);
        } elseif (isset($content->options)) {
            foreach ($content->options as &$option) {
                if (isset($option->title) && $option->title !== null) {
                    $option->title = $this->languageService->translate($option->title);
                }
                
                if (isset($option->default->thumbnail) && $option->default->thumbnail !== null) {
                    $option->default->thumbnail = $this->languageService->translate($option->default->thumbnail);
                }
                
                if (isset($option->thumbnail) && $option->thumbnail !== null) {
                    $option->thumbnail = $this->languageService->translate($option->thumbnail);
                }
                
                if (isset($option->label) && $option->label !== null) {
                    $option->label = $this->languageService->translate($option->label);
                }
                
                if (isset($option->options)) {
                    $this->translateCategories($option);
                }
                
                if (isset($option->default) && is_array($option->default)) {
                    foreach ($option->default as &$default) {
                        $default->label = $this->languageService->translate($default->label);
                    }
                }
            }
            unset($option);
        }
    }
    
    
    /**
     * @return string
     */
    protected function absolutePathForThumbnail(): string
    {
        $webPath  = (ENABLE_SSL ? HTTPS_SERVER : HTTP_SERVER) . DIR_WS_CATALOG . 'themes/' . $this->id() . '/'
                    . $this->thumbnail();
        $filePath = $this->path() . $this->thumbnail();
        
        return file_exists($filePath) && $this->thumbnail() !== '' ? $webPath : $this->fallbackImage();
    }
    
    
    /**
     * @return string
     */
    protected function fallbackImage(): string
    {
        return (ENABLE_SSL ? HTTPS_SERVER : HTTP_SERVER) . DIR_WS_CATALOG . 'images/theme_chooser_fallback.png';
    }
    
    /**
     * @return ConfigurationCategory
     */
    public function styles(): ?ConfigurationCategory
    {
        return $this->styles;
    }
    
}
