<?php
/**
 * Created by PhpStorm.
 * User: j.fischer
 * Date: 26.03.19
 * Time: 16:29
 */

namespace Gambio\StyleEdit\Core\Components\Theme\Repositories;

use Exception;
use Gambio\StyleEdit\Core\Components\Theme\CacheWriterInterface;
use Gambio\StyleEdit\Core\Components\Theme\Entities\ThemeConfiguration;
use Gambio\StyleEdit\Core\Repositories\BasicFileRepository;
use Gambio\StyleEdit\Core\SingletonPrototype;
use Gambio\StyleEdit\Core\TranslatedException;
use ThemeContentsParser;
use ZipArchive;

/**
 * Class ThemeRepository
 * @package Gambio\StyleEdit\Core\Components\Theme
 */
class StyleEditThemeRepository extends BasicFileRepository
{
    
    
    /**
     * Allowed file type for uploading
     */
    private const ALLOWED_FILETYPE = 'application/zip';
    
    /**
     * @var \Gambio\StyleEdit\Core\Components\Theme\Entities\ThemeConfigurationCollection
     */
    protected $themesList;
    
    /**
     * @var \Gambio\StyleEdit\Core\Components\Theme\Repositories\ThemeConfigurationRepository
     */
    protected $configurationRepository;
    
    /**
     * @var \ContentWriteService
     */
    protected $contentWriteService;
    
    
    /**
     * StyleEditThemeRepository constructor.
     *
     * @param \ContentWriteServiceInterface $contentWriteService
     */
    public function __construct(\ContentWriteServiceInterface $contentWriteService)
    {
        $this->contentWriteService = $contentWriteService;
    }
    
    
    /**
     * @param \Gambio\StyleEdit\Core\Components\Theme\Entities\ThemeConfiguration $theme
     *
     * @throws \Exception
     */
    public function delete(ThemeConfiguration $theme): void
    {
        if (file_exists($theme->path() . 'preview.json')) {
            $preview       = $this->jsonFileIO()->read($theme->path() . 'preview.json');
            $publishedPath = SHOP_ROOT . $preview->publishPath;
            
            if (is_dir($publishedPath)) {
                $this->recursive_rmdir($publishedPath);
            }
        }
        
        $this->recursive_rmdir($theme->path());
    }
    
    
    /**
     * @param string $src
     */
    public function recursive_rmdir(string $src)
    {
        if (empty($src)) {
            return;
        }
        
        $dir = opendir($src);
        if ($dir) {
            try {
                while (false !== ($file = readdir($dir))) {
                    if (($file !== '.') && ($file !== '..')) {
                        $full = $src . DIRECTORY_SEPARATOR . $file;
                        
                        if (is_dir($full)) {
                            $this->recursive_rmdir($full);
                        } else {
                            unlink($full);
                        }
                    }
                }
            }
            finally {
                closedir($dir);
            }
            if (is_dir($src)) {
                rmdir($src);
            }
        }
    }
    
    
    /**
     * Validates and extracts uploaded themes
     *
     * @param string $data must contain zip file
     *
     * @throws TranslatedException
     * @throws \Exception
     */
    public function save($data): void
    {
        $zipTmp = $this->storeZipFromStringToTmpDirectory($data);
        try {
            $zip = new ZipArchive;
            
            if ($zip->open($zipTmp) === true) {
                $tmpDirList         = glob($this->configuration()->tmpFolderPath() . '/*', GLOB_ONLYDIR);
                $extractedDirectory = '';
                try {
                    if ($zip->extractTo($this->configuration()->tmpFolderPath())) {
                        $extractedDirectories = array_diff(glob($this->configuration()->tmpFolderPath() . '*',
                                                                GLOB_ONLYDIR),
                                                           $tmpDirList);
                        
                        $extractedDirectory = $this->validateExtractedContent($extractedDirectories, $zipTmp);
                        
                        //  validating the uploaded configuration file
                        
                        $uploadedConfiguration = $this->loadConfigFromDisk($extractedDirectory);
                        
                        //  Does a already installed theme have the same themeId (directory name)
                        if ($this->themeIsAlreadyInstalled($extractedDirectory, $uploadedConfiguration)) {
                            throw new Exception('Theme is already installed');
                        }
                        
                        if ($zip->extractTo($this->configuration()->themesFolderPath())) {
                            $this->installContentsFromThemeJson($extractedDirectory);
                            
                            // Deactivate uploaded theme
                            $themeConfiguration = ThemeConfiguration::ThemeConfigurationWithJsonAndThemePath($uploadedConfiguration,
                                                                                                             $this->configuration()
                                                                                                                 ->themesFolderPath()
                                                                                                             . $uploadedConfiguration->id);
                            $this->configurationRepository()->deactivateTheme($themeConfiguration);
                            
                            return;
                        }
                    }
                }
                finally {
                    $this->recursive_rmdir($extractedDirectory);
                }
            }
        }
        finally {
            if (file_exists($zipTmp)) {
                unlink($zipTmp);
            }
        }
        
        throw new TranslatedException('INVALID_FILE_CONTENT', [self::ALLOWED_FILETYPE]);
    }
    
    
    /**
     * Validates Zip string and stores a valid .zip file to the temporary directory
     *
     * @param string $zip must contain zip file
     *
     * @return string path to zip file in tmp directory
     *
     * @throws TranslatedException
     * @throws \Exception
     */
    private function storeZipFromStringToTmpDirectory($zip)
    {
        //  An empty string means there was no uploaded file
        if ($zip === '') {
            throw new TranslatedException('THEME_UPLOAD_MISSING_FILE');
        }
        
        $regexPattern = '/^Content-Type:\s' . preg_quote(self::ALLOWED_FILETYPE, '/') . '/m';
        
        if (preg_match_all($regexPattern, $zip) !== 1) {
            throw new TranslatedException('INVALID_FILE_TYPE', [self::ALLOWED_FILETYPE]);
        }
        
        $tmpZipName = 'uploadedTheme' . date('Y-m-d_H-i-s') . '.zip';
        $tmpZipPath = $this->configuration()->themesFolderPath() . $tmpZipName;
        
        $zip = substr($zip, strpos($zip, 'PK'));
        
        if (!file_put_contents($tmpZipPath, $zip, 0777)) {
            throw new TranslatedException('INSUFFICIENT_PERMISSIONS', [str_replace($tmpZipName, '', $tmpZipPath)]);
        }
        
        $archive = zip_open($tmpZipPath);
        
        //  Is the stored zip file readable
        if (!is_resource($archive) || !is_readable($tmpZipPath)) {
            throw new TranslatedException('FILE_CANT_BE_READ');
        }
        
        zip_close($archive);
        
        return $tmpZipPath;
    }
    
    
    /**
     * @param string[] $extractedDirectories
     * @param string   $zipTmp
     *
     * @return string
     *
     * @throws TranslatedException
     * @throws \Exception
     */
    protected function validateExtractedContent($extractedDirectories, $zipTmp)
    {
        if (count($extractedDirectories) !== 1) {
            if (count($extractedDirectories) > 1) {
                while (count($extractedDirectories)) {
                    $this->recursive_rmdir(array_shift($extractedDirectories));
                }
            }
            unlink($zipTmp);
            throw new TranslatedException('INVALID_FILE_CONTENT', [self::ALLOWED_FILETYPE]);
        }
        
        return array_shift($extractedDirectories);
    }
    
    
    /**
     * @param string    $path                  Path to a temporary directory that ends
     *                                         with the directory name of a about to be installed theme
     * @param \stdClass $uploadedConfiguration ThemeConfiguration of temporary theme
     *
     * @return bool should the theme be installed?
     * @throws \Exception
     */
    protected function themeIsAlreadyInstalled($path, $uploadedConfiguration)
    {
        $directoryName = explode(DIRECTORY_SEPARATOR, $path);
        $directoryName = array_pop($directoryName);
        $themePath     = $this->configuration()->themesFolderPath() . DIRECTORY_SEPARATOR . $directoryName;
        
        $themeInstalled = true;
        
        try {
            $installedTheme = $this->loadConfigFromDisk($themePath);
        } catch (TranslatedException $exception) {
            $themeInstalled = false;
        } catch (Exception $e) {
            $themeInstalled = false;
        }
        
        if (false && $themeInstalled && $installedTheme->version < $uploadedConfiguration->version) {
            $this->recursive_rmdir($themePath);
            
            return false;
        }
        
        return $themeInstalled;
    }
    
    
    /**
     * @param string $extractedDirectory
     *
     * @throws TranslatedException
     * @throws \Gambio\StyleEdit\Core\InvalidDirectoryException
     */
    protected function installContentsFromThemeJson(string $extractedDirectory): void
    {
        $extractedDirectorySplitBySeparator = explode(DIRECTORY_SEPARATOR, $extractedDirectory);
        $themeName                          = array_pop($extractedDirectorySplitBySeparator);
        $themePath                          = $this->configuration()->themesFolderPath() . DIRECTORY_SEPARATOR
                                              . $themeName;
        
        $uploadedConfiguration = $this->loadConfigFromDisk($themePath);
        
        if (isset($uploadedConfiguration->contents)
            && get_class($uploadedConfiguration->contents) === \stdClass::class) {
            $themeContents = ThemeContentsParser::parse($uploadedConfiguration->contents);
            
            if ($themeContents->infoElements()->count()) {
                foreach ($themeContents->infoElements()->getArray() as $infoElement) {
                    $this->contentWriteService->storeInfoElementContent($infoElement);
                }
            }
            
            if ($themeContents->infoPages()->count()) {
                foreach ($themeContents->infoPages()->getArray() as $infoPage) {
                    $this->contentWriteService->storeInfoPageContent($infoPage);
                }
            }
            
            if ($themeContents->linkPages()->count()) {
                foreach ($themeContents->linkPages()->getArray() as $linkPage) {
                    $this->contentWriteService->storeLinkPageContent($linkPage);
                }
            }
        }
    }
    
    
    /**
     * @return \Gambio\StyleEdit\Core\Components\Theme\Repositories\ThemeConfigurationRepository
     * @throws \Exception
     */
    public function configurationRepository()
    {
        if ($this->configurationRepository === null) {
            $this->configurationRepository = SingletonPrototype::instance()->get(ThemeConfigurationRepository::class);
            if (!$this->configurationRepository) {
                throw new Exception('ConfigurationRepository is not initialized');
            }
        }
        
        return $this->configurationRepository;
    }
    
    
    /**
     * @param \Gambio\StyleEdit\Core\Components\Theme\Entities\ThemeConfiguration $source
     * @param \Gambio\StyleEdit\Core\Components\Theme\Entities\ThemeConfiguration $destination
     *
     * @return \Gambio\StyleEdit\Core\Components\Theme\Entities\ThemeConfiguration
     * @throws \Exception
     */
    public function copyTo(ThemeConfiguration $source, ThemeConfiguration $destination)
    {
        $destinationPath = $this->configuration()->themesFolderPath() . DIRECTORY_SEPARATOR . $destination->id();
        if (!is_dir($destinationPath) && !mkdir($destinationPath, 0777) && !is_dir($destinationPath)) {
            throw new \Exception("Can't create new theme directories");
        }
        $this->recursive_copy($source->path(), $destinationPath);
        
        $this->configurationRepository()
            ->save(ThemeConfiguration::ThemeConfigurationWithJsonAndThemePath((object)$destination->toArray(),
                                                                              $destinationPath));
        
        return $this->configurationRepository()->getById($destination->id());
    }
    
    
    /**
     * Copy the contents of a directory recursively into another
     *
     * @param string $source      path
     * @param string $destination path
     *
     * @return bool
     */
    public function recursive_copy($source, $destination)
    {
        if (is_link($source)) {
            return symlink(readlink($source), $destination);
        }
        
        if (is_file($source)) {
            return copy($source, $destination);
        }
        //check is_dir twice to avoid race problems
        if (is_dir($destination) || mkdir($destination) || is_dir($destination)) {
            
            $dir = dir($source);
            try {
                while (false !== $entry = $dir->read()) {
                    //  Skip pointers
                    if ($entry == '.' || $entry == '..') {
                        continue;
                    }
                    
                    $this->recursive_copy($source . DIRECTORY_SEPARATOR . $entry,
                                          $destination . DIRECTORY_SEPARATOR . $entry);
                }
            }
            finally {
                $dir->close();
            }
        }
        
        // Clean up
        
        return true;
    }
    
    
}