<?php
/* --------------------------------------------------------------
   ThemeConfigurationRepository.php 2019-07-09
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2016 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

namespace Gambio\StyleEdit\Core\Components\Theme\Repositories;

use Exception;
use Gambio\StyleEdit\Core\Components\Theme\Entities\ThemeConfiguration;
use Gambio\StyleEdit\Core\Components\Theme\Entities\ThemeConfigurationCollection;
use Gambio\StyleEdit\Core\InvalidDirectoryException;
use Gambio\StyleEdit\Core\Repositories\BasicFileRepository;
use Gambio\StyleEdit\Core\Services\SettingsService;
use Gambio\StyleEdit\Core\SingletonPrototype;
use Gambio\StyleEdit\Core\TranslatedException;
use ReflectionException;

/**
 * Class ConfigurationRepository
 * @package Gambio\StyleEdit\Core\Components\Theme
 */
class ThemeConfigurationRepository extends BasicFileRepository
{
    /**
     * @var SettingsService
     */
    protected $settingsService;
    
    /**
     * @var \Gambio\StyleEdit\Core\Components\Theme\Entities\ThemeConfigurationCollection
     */
    private $ThemeConfigurationCollection;
    
    
    /**
     * @return SettingsService
     * @throws ReflectionException
     */
    protected function settingsService(): SettingsService
    {
        if ($this->settingsService === null) {
            $this->settingsService = SingletonPrototype::instance()->get(SettingsService::class);
        }
        
        return $this->settingsService;
    }
    
    
    /**
     * @return \Gambio\StyleEdit\Core\Components\Theme\Entities\ThemeConfigurationCollection
     * @throws \Exception
     */
    public function get()
    {
        $result              = new ThemeConfigurationCollection();
        $themeDirectoryPaths = $this->getThemeDirectoryPaths();
        
        foreach ($themeDirectoryPaths as $themePath) {
            $themeConfig = $this->loadConfigFromDisk($themePath);
            $theme       = ThemeConfiguration::ThemeConfigurationWithJsonAndThemePath($themeConfig, $themePath);
            
            if ($result->hasKey($theme->id())) {
                throw new TranslatedException('DUPLICATED_THEME_ID',
                                              [$theme->title(), $result->get($theme->id())->title()]);
            }
            $result->add($theme);
        }
        
        //map theme hierarchy
        foreach ($result->elements() as &$theme) {
            if ($theme->extendsOf()) {
                if (!$result->hasKey($theme->extendsOf())) {
                    throw new TranslatedException('INVALID_PARENT_THEME_ID', [$theme->title(), $theme->extendsOf()]);
                }
                $result->get($theme->extendsOf())->children()[] = $theme->id();
            }
        }
        unset ($theme);
        
        return $result;
    }
    
    
    /**
     * @return array
     * @throws \Exception
     */
    private function getThemeDirectoryPaths()
    {
        return $this->jsonFileIO()->listDirectory($this->configuration()->themesFolderPath());
    }
    
    
    /**
     * @param string $themeId
     *
     * @return ThemeConfiguration
     * @throws ReflectionException
     * @throws TranslatedException
     */
    public function getById(string $themeId): ThemeConfiguration
    {
        try {
            $themePath   = $this->configuration()->themesFolderPath() . DIRECTORY_SEPARATOR . $themeId;
            $themeConfig = $this->loadConfigFromDisk($themePath);
            
            if (isset($themeConfig->colorPalette) && is_array($themeConfig->colorPalette)) {
                $colorPalette = [];
                foreach ($themeConfig->colorPalette as $variableName) {
                    $colorPalette[$variableName] = $this->settingsService()->findSettingValueByName($variableName);
                }
                $themeConfig->colorPalette = $colorPalette;
            }
            
            $themeConfig->children = $this->get()->get($themeId)->children();
            
            $theme = ThemeConfiguration::ThemeConfigurationWithJsonAndThemePathExtended($themeConfig, $themePath);
        } catch (InvalidDirectoryException $e) {
            throw new Exception('Invalid theme id: ' . $themeId);
        }
        
        return $theme;
    }
    
    
    /**
     * @return \Gambio\StyleEdit\Core\Components\Theme\Entities\ThemeConfigurationCollection
     * @throws \Exception
     */
    private function ThemeConfigurationCollection()
    {
        if ($this->ThemeConfigurationCollection === null) {
            $this->ThemeConfigurationCollection = SingletonPrototype::instance()
                ->get(ThemeConfigurationCollection::class);
            if (!$this->ThemeConfigurationCollection) {
                throw new Exception('ThemeConfigurationCollection was not initialized');
            }
        }
        
        return $this->ThemeConfigurationCollection;
    }
    
    
    /**
     * @param ThemeConfiguration $themeConfiguration
     *
     * @return void
     * @throws Exception
     */
    public function save(ThemeConfiguration $themeConfiguration): void
    {
        $data           = $themeConfiguration->toArray();
        $data['active'] = $themeConfiguration->active();
        $this->patch($themeConfiguration, $data);
    }
    
    
    /**
     * @param ThemeConfiguration $theme
     * @param                    $data
     *
     * @throws \Exception
     */
    public function patch(ThemeConfiguration $theme, $data)
    {
        $configuration = $this->loadConfigFromDisk($theme->path());
        
        foreach ($data as $key => $value) {
            $configuration->$key = $value;
        }
        
        if (isset($configuration->active) && !$configuration->active) {
            unset($configuration->active);
        }
    
        if (empty($configuration->extends)) {
            unset($configuration->extends);
        }
        
        $this->saveConfigToDisk($configuration, $theme->path());
    }
    
    
    /**
     * @param ThemeConfiguration $theme
     *
     * @throws InvalidDirectoryException
     * @throws TranslatedException
     */
    public function deactivateTheme(ThemeConfiguration $theme)
    {
        $config = $this->loadConfigFromDisk($theme->path());
        if (isset($config->active)) {
            unset ($config->active);
            $this->saveConfigToDisk($config, $theme->path());
        }
    }
    
    
    /**
     * @return bool|mixed
     */
    public static function create()
    {
        return SingletonPrototype::instance()->get(static::class);
    }
    
}

