<?php
/* --------------------------------------------------------------
  VariantRepository.php 2019-07-19
  Gambio GmbH
  http://www.gambio.de
  Copyright (c) 2019 Gambio GmbH
  Released under the GNU General Public License (Version 2)
  [http://www.gnu.org/licenses/gpl-2.0.html]
  --------------------------------------------------------------*/

namespace Gambio\StyleEdit\Core\Components\Variant\Repositories;

use Exception;
use FileNotFoundException;
use FilesystemAdapter;
use Gambio\StyleEdit\Core\Components\Theme\Entities\Interfaces\CurrentThemeInterface;
use Gambio\StyleEdit\Core\Components\Theme\Entities\ThemeConfiguration;
use Gambio\StyleEdit\Core\Repositories\ConfigurationRepository;
use Gambio\StyleEdit\Core\SingletonPrototype;
use Gambio\StyleEdit\StyleEditConfiguration;
use InvalidArgumentException;
use ReflectionException;
use stdClass;

/**
 * Class VariantRepository
 *
 * @package Gambio\StyleEdit\Core\Components\Variant\Repositories
 */
class VariantRepository
{
    /**
     * @var ThemeConfiguration
     */
    protected $currentTheme;
    
    /**
     * @var FilesystemAdapter
     */
    protected $themeFileSystem;
    /**
     * @var StyleEditConfiguration
     */
    protected $configuration;
    
    
    /**
     * @return ThemeConfiguration
     * @throws Exception
     */
    protected function currentTheme(): ThemeConfiguration
    {
        if ($this->currentTheme === null) {
            
            $this->currentTheme = SingletonPrototype::instance()->get(CurrentThemeInterface::class);
            
            if (!$this->currentTheme) {
                
                throw new Exception('CurrentTheme was not initialized');
            }
        }
        
        return $this->currentTheme;
    }
    
    
    /**
     * @return FilesystemAdapter
     * @throws ReflectionException
     */
    protected function themeFileSystem(): FilesystemAdapter
    {
        if ($this->themeFileSystem === null) {
            
            $this->themeFileSystem = SingletonPrototype::instance()->get(FilesystemAdapter::class,
                                                                         $this->configuration->themesFolderPath());
        }
        
        return $this->themeFileSystem;
    }
    
    
    /**
     * VariantRepository constructor.
     *
     * @param StyleEditConfiguration $configuration
     */
    public function __construct(StyleEditConfiguration $configuration)
    {
        $this->configuration = $configuration;
    }
    
    
    /**
     * @param string $directory relative path to variant option
     *
     * @return stdClass
     *
     * @throws ReflectionException
     * @throws Exception
     */
    public function loadVariantJson(string $directory): stdClass
    {
        $variantJsonPath = $this->currentTheme()->id() . DIRECTORY_SEPARATOR . $directory;
        
        if ($variantJsonPath[strlen($variantJsonPath) - 1] !== DIRECTORY_SEPARATOR) {
            
            $variantJsonPath .= DIRECTORY_SEPARATOR;
        }
        
        $variantJsonPath .= 'variant.json';
        
        if ($this->themeFileSystem()->has($variantJsonPath) === false) {
            
            throw new Exception($variantJsonPath . ' does not exist');
        }
        
        $jsonString = $this->themeFileSystem()->read($variantJsonPath);
        
        $variantJson = json_decode($jsonString, false);
        
        if (json_last_error() !== JSON_ERROR_NONE) {
            
            throw new Exception($variantJsonPath . ' does not hold a valid json file');
        }
        
        $this->initializeValuesFromVariantSettingsJson($variantJson, $directory);
        
        return $variantJson;
    }
    
    
    /**
     * @param string $directory relative path to variant option
     *
     * @return bool
     * @throws ReflectionException
     */
    public function variantJsonExists(string $directory): bool
    {
        $variantJsonPath = $this->currentTheme()->id() . DIRECTORY_SEPARATOR . $directory;
        
        if ($variantJsonPath[strlen($variantJsonPath) - 1] !== DIRECTORY_SEPARATOR) {
            
            $variantJsonPath .= DIRECTORY_SEPARATOR;
        }
        
        $variantJsonPath .= 'variant.json';
        
        return $this->themeFileSystem()->has($variantJsonPath);
    }
    
    
    /**
     * @param stdClass $categories
     * @param string   $directory relative path to variant option
     *
     * @throws Exception
     */
    protected function initializeValuesFromVariantSettingsJson(stdClass $categories, string $directory): void
    {
        if (isset($categories->categories)) {
            
            foreach ($categories->categories as $category) {
                
                $this->initializeValuesFromVariantSettingsJson($category, $directory);
            }
        } elseif (isset($categories->fieldsets)) {
            
            $this->initializeValueInFieldsets($categories->fieldsets, $directory);
        }
    }
    
    
    /**
     * @param array  $fieldsets
     * @param string $directory relative path to variant option
     *
     * @throws Exception
     */
    protected function initializeValueInFieldsets(array $fieldsets, string $directory): void
    {
        $configuration = ConfigurationRepository::instance()->getVariantConfigurationCollection($directory);
        
        foreach ($fieldsets as $fieldset) {
            
            foreach ($fieldset->options as $option) {
                
                $setValue = $configuration !== false;
                
                if ($setValue) {
                    
                    try {
                        $option->value = $configuration->getValue($option->id)->value();
                    } catch (InvalidArgumentException $argumentException) {
                        $setValue = false;
                    }
                }
                
                if ($setValue === false) {
                    $option->value = $option->default;
                }
            }
        }
    }
}