<?php
/* --------------------------------------------------------------
   VariantService.php 2019-06-06
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2018 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

namespace Gambio\StyleEdit\Core\Components\Variant\Services;

use Exception;
use Gambio\StyleEdit\Core\Components\Theme\Entities\Interfaces\CurrentThemeInterface;
use Gambio\StyleEdit\Core\Components\Variant\Repositories\VariantRepository;
use Gambio\StyleEdit\Core\SingletonPrototype;
use Gambio\StyleEdit\Core\Components\Theme\Entities\ThemeConfiguration;
use Gambio\StyleEdit\Core\Components\Variant\Entities\VariantOption;
use Gambio\StyleEdit\StyleEditConfiguration;
use LanguageService;
use ReflectionException;
use stdClass;

/**
 * Class VariantService
 * @package Gambio\StyleEdit\Core\Components\Variant\Services
 */
class VariantService
{
    /**
     * @var \Gambio\StyleEdit\StyleEditConfiguration
     */
    protected $settings;
    
    /**
     * @var \Gambio\StyleEdit\Core\Components\Theme\Entities\ThemeConfiguration
     */
    protected $currentTheme;
    
    /**
     * @var VariantRepository
     */
    protected $repository;
    
    /**
     * @var LanguageService
     */
    protected $languageService;
    
    
    /**
     * @param \Gambio\StyleEdit\Core\Components\Variant\Entities\VariantOption $variant
     *
     * @throws \Exception
     */
    public function activateVariant(VariantOption $variant): void
    {
        $sourceDir = $this->currentTheme()->path() . $variant->dir();
        $destDir   = $this->currentTheme()->path() . DIRECTORY_SEPARATOR . '/variant';
        $this->recursive_copy($sourceDir, $destDir);
    }
    
    
    /**
     * @return bool|\Gambio\StyleEdit\StyleEditConfiguration|mixed
     * @throws \Exception
     */
    public function settings()
    {
        if ($this->settings === null) {
            $this->settings = SingletonPrototype::instance()->get(StyleEditConfiguration::class);
            if (!$this->settings) {
                throw new Exception('settings was not initialized');
            }
        }
        
        return $this->settings;
    }
    
    
    /**
     * @return ThemeConfiguration
     * @throws \Exception
     */
    public function currentTheme()
    {
        if ($this->currentTheme === null) {
            $this->currentTheme = SingletonPrototype::instance()->get(CurrentThemeInterface::class);
            if (!$this->currentTheme) {
                throw new Exception('CurrentTheme was not initialized');
            }
        }
        
        return $this->currentTheme;
    }
    
    
    /**
     * @return VariantRepository
     * @throws ReflectionException
     */
    public function repository(): VariantRepository
    {
        if ($this->repository === null) {
            
            $this->repository = SingletonPrototype::instance()->get(VariantRepository::class);
        }
        
        return $this->repository;
    }
    
    
    /**
     * @param array $category
     *
     * @return array $category with variant categories
     *
     * @throws ReflectionException
     */
    public function addVariantCategoriesToCategory(array $category): array
    {
        $variantCategories = [];
        
        foreach ($category as &$categories) {
            
            foreach ($categories->fieldsets as &$fieldset) {
                
                foreach ($fieldset->options as &$options) {
                    
                    foreach ($options->options as &$option) {
                        
                        $option->dir = 'variants/' . $options->id . '/' . $option->id;
                        
                        if ($this->repository()->variantJsonExists($option->dir)) {
                            
                            $variantJson           = $this->repository()->loadVariantJson($option->dir);
                            $variantIdentification = $options->id . '/' . $option->id;
                            
                            $this->setForProperties($variantJson, $variantIdentification);
                            $this->translateVariantProperties($variantJson);
                            
                            $variantCategories[] = $variantJson;
                        }
                    }
                }
            }
        }
        
        unset($categories, $fieldset, $options, $option);
        
        return array_merge($category, $variantCategories);
    }
    
    
    /**
     * @param stdClass $variantJson
     * @param string   $variantIdentification
     */
    protected function setForProperties(stdClass $variantJson, string $variantIdentification): void
    {
        $variantJson->for = $variantIdentification;
        
        if (isset($variantJson->categories)) {
            
            foreach ($variantJson->categories as $category) {
                
                $this->setForProperties($category, $variantIdentification);
            }
        }
        
        if (isset($variantJson->fieldsets)) {
            
            foreach ($variantJson->fieldsets as $fieldset) {
                
                $fieldset->for = $variantIdentification;
                
                if (isset($fieldset->options)) {
                    
                    foreach ($fieldset->options as $option) {
                        
                        $option->for = $variantIdentification;
                    }
                }
            }
        }
    }
    
    
    /**
     * @param stdClass $variantJson
     *
     * @throws ReflectionException
     */
    protected function translateVariantProperties(stdClass $variantJson): void
    {
        $languageService = $this->languageService();
        
        $variantJson->title = $languageService->translate($variantJson->title);
        
        if (isset($variantJson->categories)) {
            
            foreach ($variantJson->categories as $category) {
                
                $this->translateVariantProperties($category);
            }
        }
        
        if (isset($variantJson->fieldsets)) {
            
            foreach ($variantJson->fieldsets as $fieldset) {
                
                $fieldset->title = $languageService->translate($fieldset->title);
                
                foreach ($fieldset->options as $option) {
                    
                    $option->label = $languageService->translate($option->label);
                }
            }
        }
    }
    
    
    /**
     * @return LanguageService
     * @throws ReflectionException
     */
    protected function languageService(): LanguageService
    {
        if ($this->languageService === null) {
            
            $this->languageService = SingletonPrototype::instance()->get(LanguageService::class);
        }
        
        return $this->languageService;
    }
}