<?php
/*--------------------------------------------------------------------------------------------------
    ConfigurationRepository.php 2019-06-06
    Gambio GmbH
    http://www.gambio.de
    Copyright (c) 2016 Gambio GmbH
    Released under the GNU General Public License (Version 2)
    [http://www.gnu.org/licenses/gpl-2.0.html]
    --------------------------------------------------------------------------------------------------
 */

namespace Gambio\StyleEdit\Core\Repositories;

use Exception;
use Gambio\StyleEdit\Core\Components\Theme\Entities\Interfaces\CurrentThemeInterface;
use Gambio\StyleEdit\Core\Components\Theme\Entities\ThemeConfiguration;
use Gambio\StyleEdit\Core\Components\Variant\Entities\VariantOption;
use Gambio\StyleEdit\Core\Options\Entities\OptionInterface;
use Gambio\StyleEdit\Core\Repositories\Entities\Configuration;
use Gambio\StyleEdit\Core\Repositories\Entities\ConfigurationCollection;
use Gambio\StyleEdit\Core\SingletonPrototype;
use Gambio\StyleEdit\StyleEditConfiguration;
use ReflectionException;
use stdClass;

/**
 * Class ConfigurationRepository
 * @package Gambio\StyleEdit\Core\Repositories
 */
class ConfigurationRepository extends BasicFileRepository
{
    /**
     * @var self
     */
    private static $instance;
    /**
     * @var ConfigurationCollection
     */
    private $configurationList = null;
    
    /**
     * @var string
     */
    private $themeId;
    
    
    /**
     * ConfigurationRepository constructor.
     *
     * @param  $themeConfig
     */
    public function __construct(CurrentThemeInterface $themeConfig = null)
    {
        if ($themeConfig) {
            $this->themeId = $themeConfig->id();
        }
    }
    
    
    /**
     * @return ConfigurationRepository
     * @throws \Exception
     */
    public static function instance(): self
    {
        if (!self::$instance) {
            self::$instance = SingletonPrototype::instance()->get(static::class);
            if (!self::$instance) {
                throw new Exception('ConfigurationRepository was not initialized');
            }
            $currentTheme = SingletonPrototype::instance()->get(CurrentThemeInterface::class);
            self::$instance->setThemeId($currentTheme->id());
        }
        
        return self::$instance;
    }
    
    
    /**
     * @param string $themeId
     *
     * @return ConfigurationRepository
     */
    public static function createForTheme(string $themeId): self
    {
        $result = new static(null);
        $result->setThemeId($themeId);
        
        return $result;
    }
    
    
    /**
     * @param string $name
     *
     * @return Configuration
     * @throws \Exception
     */
    public function getJsonConfigurationFrom(string $name)
    {
        // Transform camelCase to dash-case because the settings.json is dash-cased and theme.json is camelCased
        $name = strtolower(preg_replace('/([a-zA-Z])(?=[A-Z])/', '$1-', $name));
        
        if ($this->configurationsList()->keyExists($name)) {
            return $this->configurationsList()->getValue($name);
        }
        
        return false;
    }
    
    
    /**
     * @param \Gambio\StyleEdit\Core\Repositories\Entities\Configuration $configuration
     *
     * @throws \Exception
     */
    public function saveJsonConfigurationFrom(Configuration $configuration)
    {
        $this->configurationsList()->setValue($configuration->id(), $configuration);
        $this->configurationsList()->sort();
        $this->saveJsonFilesToDisk($this->configurationsList(), $this->settingsFileName());
    }
    
    
    /**
     * @param \Gambio\StyleEdit\Core\Options\Entities\OptionInterface $option
     *
     * @throws \Exception
     */
    public function saveOptionToConfiguration(OptionInterface $option): void
    {
        if ($this->configurationsList()->keyExists($option->id())) {
            $jsonConfiguration = $this->configurationsList()->getValue($option->id());
            $jsonConfiguration = $jsonConfiguration->jsonSerialize();
            if (!isset($jsonConfiguration->type)) {
                $jsonConfiguration->type = $option->type();
            }
        } else {
            $jsonConfiguration        = new stdClass();
            $jsonConfiguration->name  = $option->id();
            $jsonConfiguration->type  = $option->type();
            $jsonConfiguration->group = $option->group() ?? 'bootstrap';
        }
        
        $jsonConfiguration->value = $option->value();
        
        $configuration = Configuration::createFromJson($jsonConfiguration);
        $this->configurationsList()->setValue($configuration->id(), $configuration);
        $this->saveJsonFilesToDisk($this->configurationsList(), $this->settingsFileName());
        
        // variant specific option
        if ($option->for() !== null) {
            
            [$variantId, $variantOptionId] = explode('/', $option->for());
            
            try {
                $variantJson = $this->loadJsonFilesFromDisk($this->variantSettingsFileName($variantId,
                                                                                           $variantOptionId));
            } catch (Exception $exception) { // if file was not found
                $variantJson = [];
            }
            
            $variantConfiguration = ConfigurationCollection::createFromJsonList($variantJson);
            $variantConfiguration->setValue($configuration->id(), $configuration);
            
            $this->saveJsonFilesToDisk($variantConfiguration,
                                       $this->variantSettingsFileName($variantId, $variantOptionId));
        }
    }
    
    
    /**
     * @param string $variantDirectoryPath
     *
     * @return ConfigurationCollection|false
     * @throws Exception
     */
    public function getVariantConfigurationCollection(string $variantDirectoryPath)
    {
        list($variantId, $variantOptionId) = array_slice(explode('/', $variantDirectoryPath), 1);
        
        if (file_exists($this->variantSettingsFileName($variantId, $variantOptionId))) {
            $variantJson = $this->loadJsonFilesFromDisk($this->variantSettingsFileName($variantId, $variantOptionId));
            
            return ConfigurationCollection::createFromJsonList($variantJson);
        }
        
        return false;
    }
    
    
    /**
     * @return string
     * @throws ReflectionException
     */
    private function settingsFileName(): string
    {
        $configurations = SingletonPrototype::instance()->get(StyleEditConfiguration::class);
        
        return $configurations->themesFolderPath() . $this->themeId . DIRECTORY_SEPARATOR . 'settings.json';
    }
    
    
    /**
     * @param string $variantId
     * @param string $variantOptionId
     *
     * @return string
     * @throws ReflectionException
     */
    private function variantSettingsFileName(string $variantId, string $variantOptionId): string
    {
        $configurations = SingletonPrototype::instance()->get(StyleEditConfiguration::class);
        
        $variantsDirectoryPath = $configurations->themesFolderPath() . $this->themeId . DIRECTORY_SEPARATOR
                                 . 'variants';
        
        return $variantsDirectoryPath . DIRECTORY_SEPARATOR . $variantId . DIRECTORY_SEPARATOR . $variantOptionId
               . DIRECTORY_SEPARATOR . 'settings.json';
    }
    
    
    /**
     * @param VariantOption $option
     *
     * @throws ReflectionException
     */
    public function restoreVariantSettings(VariantOption $option): void
    {
        if (!empty($option->id()) && !empty($option->value()->id())) {
            
            $variantId       = $option->id();
            $variantOptionId = $option->value()->id();
            
            try {
                $variantJson = $this->loadJsonFilesFromDisk($this->variantSettingsFileName($variantId,
                                                                                           $variantOptionId));
            } catch (Exception $exception) {
                return;
            }
            
            $variantConfiguration = ConfigurationCollection::createFromJsonList($variantJson);
            
            if ($variantConfiguration->count()) {
                
                foreach ($variantConfiguration as $variantOption) {
                    
                    $this->configurationsList()->setValue($variantOption->id(), $variantOption);
                }
                
                $this->saveJsonFilesToDisk($this->configurationsList(), $this->settingsFileName());
            }
        }
    }
    
    
    /**
     * @return array|mixed
     * @throws ReflectionException
     */
    private function loadConfigurationsObject()
    {
        return $this->loadJsonFilesFromDisk($this->settingsFileName());
    }
    
    
    /**
     * @return array|mixed
     * @throws ReflectionException
     */
    public function hasConfiguration()
    {
        return $this->jsonFileIO()->exists($this->settingsFileName());
    }
    
    
    /**
     * @return ConfigurationCollection
     * @throws \Exception
     */
    public function configurationsList(): ConfigurationCollection
    {
        if ($this->configurationList === null) {
            $this->configurationList = ConfigurationCollection::createFromJsonList($this->loadConfigurationsObject());
        }
        
        return $this->configurationList;
    }
    
    
    /**
     * @param string $themeId
     */
    protected function setThemeId(string $themeId)
    {
        $this->themeId = $themeId;
    }
    
}