<?php
/* --------------------------------------------------------------
  ThemeValidator.php 2019-09-18
  Gambio GmbH
  http://www.gambio.de
  Copyright (c) 2019 Gambio GmbH
  Released under the GNU General Public License (Version 2)
  [http://www.gnu.org/licenses/gpl-2.0.html]
  --------------------------------------------------------------*/

namespace Gambio\StyleEdit\Core\Services;

use Exception;
use FileNotFoundException;
use FilesystemAdapter;
use Gambio\StyleEdit\Core\SingletonPrototype;
use stdClass;

/**
 * Class ThemeValidator
 *
 * Checks if a theme can be opened in StyleEdit4.
 * 2 files are needed for that.
 *
 * settings.json
 * * * * *  Must be a valid json
 * * * * *  content must be an array
 * * * * *  can be an empty array
 * * * * *  if not empty only objects inside that array
 *
 * theme.json
 * * * * *  Must be a valid json
 * * * * *  content must be an object
 * * * * *  content must have an id
 * * * * *  content must have a thumbnail
 * * * * *  content must have a config object
 * * * * * * * * *  with a basics & areas object
 */
class ThemeValidator
{
    /**
     * @var string
     */
    protected $themeName;
    
    /**
     * @var FilesystemAdapter
     */
    protected $filesystemThemes;
    
    /**
     * @var bool
     */
    protected $themeCanBeOpenedInStyleEdit4 = true;
    
    
    /**
     * @param string                 $themeName
     *
     * @param FilesystemAdapter|null $adapter
     *
     * @return ThemeValidator
     * @throws FileNotFoundException
     * @throws Exception
     */
    public static function for(string $themeName, ?FilesystemAdapter $adapter = null): self
    {
        // the SingletonPrototype can only construct this if the inject method of the dependencyInjector class was executed!
        $filesystemThemes = $adapter ?? SingletonPrototype::instance()->get(FilesystemAdapter::class);
        
        return new static($themeName, $filesystemThemes);
    }
    
    
    /**
     * ThemeValidator constructor.
     *
     * @param string            $themeName
     * @param FilesystemAdapter $filesystemThemes
     *
     * @throws FileNotFoundException
     */
    public function __construct(string $themeName, FilesystemAdapter $filesystemThemes)
    {
        $this->themeName        = $themeName;
        $this->filesystemThemes = $filesystemThemes;
        
        $this->validateSettingsJson();
        
        if ($this->themeCanBeOpenedInStyleEdit4) {
            
            $this->validateThemeJson();
        }
    }
    
    
    /**
     * @throws FileNotFoundException
     */
    protected function validateSettingsJson(): void
    {
        $settingsJsonPath = $this->themeName . DIRECTORY_SEPARATOR . 'settings.json';
        
        //  every theme needs a settings.json
        if (!$this->filesystemThemes->has($settingsJsonPath)) {
            
            $this->themeCanBeOpenedInStyleEdit4 = false;
            
            return;
        }
        
        $settingsJsonContent = $this->filesystemThemes->read($settingsJsonPath);
        $settingsJson        = json_decode($settingsJsonContent, false);
        
        //  content has to be an array
        if (!is_array($settingsJson)) {
            
            $this->themeCanBeOpenedInStyleEdit4 = false;
            
            return;
        }
        
        //  a settings.json can be empty array
        if (empty($settingsJson)) {
            return;
        }
        
        // everything inside the array must be stdClass
        foreach ($settingsJson as $option) {
            
            if (!is_a($option, stdClass::class)) {
                
                $this->themeCanBeOpenedInStyleEdit4 = false;
            }
        }
    }
    
    
    /**
     *
     * @throws FileNotFoundException
     */
    protected function validateThemeJson(): void
    {
        $themeJsonPath = $this->themeName . DIRECTORY_SEPARATOR . 'theme.json';
        
        if (!$this->filesystemThemes->has($themeJsonPath)) {
            
            $this->themeCanBeOpenedInStyleEdit4 = false;
            
            return;
        }
        
        $themeJsonContent = $this->filesystemThemes->read($themeJsonPath);
        $themeJson        = json_decode($themeJsonContent, false);
        
        if (!$themeJson instanceof stdClass
            || !isset($themeJson->id, $themeJson->thumbnail, $themeJson->config, $themeJson->config->basics, $themeJson->config->areas)
            || !$themeJson->config->basics instanceof stdClass
            || !$themeJson->config->areas instanceof stdClass) {
            
            $this->themeCanBeOpenedInStyleEdit4 = false;
        }
    }
    
    
    /**
     * @return bool
     */
    public function canBeOpenedInStyleEdit4(): bool
    {
        return $this->themeCanBeOpenedInStyleEdit4;
    }
}